<?php
namespace Plugin\Routing;

use Plugin\Exception\Error;
use App\Config\Settings;

class Router {
	
	private static $rt;
	
	private $routes;
	private $uri;
	private $currentRoute;
	
	private static function initialize(){
		if (self::$rt){
			return ;
		}
		$rt = new self();
		
		$app = Settings::getAppsInfo();
		$routePaths = $rt->getAppPaths($app);
		
		$allRoutes = array();
		
		foreach ($routePaths as $path){
			$filepath = ROOT_DIR."{$path}";
			if (file_exists($filepath)){
				include($filepath); // $routes variable is in the include
				$allRoutes = array_merge($allRoutes,$routes);
			}
		}
		
		if (empty($allRoutes)){
			Error::throw500("There are no routes availabe. Go to src/{Your App}/{Section}/Config/routes.php");
		}
		
		$rt->setUri($_SERVER["REQUEST_URI"]);
		$rt->setRoutes($allRoutes);
		
		self::$rt = $rt;
	}
	
	public static function getRoute(){
		self::initialize();
		$rt = self::$rt;
		$rt->processRoute();
		return $rt->getCurrentRoute();
	}
	
	public function setUri($uri){
		$this->uri = $uri;
	}
	
	public function setRoutes($routes){
		$this->routes = $routes;
	}
	
	public function getCurrentRoute(){
		return $this->currentRoute;
	}
	
	public function processRoute(){
		$path = explode("/",$this->uri);
		$path_length = count($path);
		
		$match = false;
		$theMatch = null;
		$vars = array();
		$mat = array();
		foreach ($this->routes as $l) {
			
			foreach($l as $name => $config){
				
				if ($match){
					break;
				}
				$invars = array();
				//echo "$name\n";
				$pattern = $l['_pattern'];
				$pattern_path = explode("/",$pattern);
		
				$c = 1;
				$patterna = preg_replace_callback("/(\{.*\})/U", function ($match) use (&$invars){
					$invars[] = str_replace("{", "", str_replace("}", "", $match[1]));
					return "(.*)";
				}, $pattern);
				//print_r($vars);
				if (preg_match("~^$patterna$~", $this->uri,$mematch)){
					array_shift($mematch);
					$values = array();
					$cTotal = count($mematch);
					for($i=0;$i<$cTotal;$i++){
						$values[$invars[$i]] = $mematch[$i];
					}
					$theMatch = array ('name'=> $name,"config"=>$config,"values"=>$values);
					$match = true;
					$this->currentRoute = $theMatch;
					$vars[] = $invars;
					unset($invars);
					break;
				}
		
			}
			if ($match){
				break;
			}
		}
		
		if ($theMatch){
			// It worked
		}else {
			Error::throw404("The current page cannot be found, Url does not match any controller");
		}
	}
	
	public static function getTemplatePath($path){
		list($app,$name,$template) = explode(":",$path);
		$filepath = "src/{$app}/{$name}/template/layout/$template";
		return $filepath;
	}
	
	
	private function getAppPaths($apps){
		$path = array();
		foreach ($apps as $app => $sections) {
			foreach($sections as $name){
				$path[] = "/src/$app/$name/Config/routes.php";
			}
		}
		return $path;
	}
}